#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from tkinter import *
import palier_1
import palier_2
import palier_3
import palier_4
#import moulinette
import random
import argparse

CELL_NUMBER = 9
CELL_SIZE = 50
MARGIN_X = 15
MARGIN_Y = 10

class Interface(Frame):
	def __init__(self, window, board, **kwargs):
		Frame.__init__(self, window, width = 768, height = 576, bg = "white", **kwargs)
		self.pack(fill = BOTH)
		self.can = Canvas(window, height = 475, width = 475, bg = "white")
		self.board = board
		self.can.bind("<Key>", self.key_pressed)
		self.can.bind("<Button-1>", self.draw_cursor)
		self.protected = 81 * [0]

		self.solve_button = Button(self, text = "Résoudre", bg = "white", command = self.solve)
		self.check_button = Button(self, text = "Valider", bg = "white", command = self.validate)
		self.generate_button = Button(self, text = "Génerer", bg = "white", command = self.generate)

		self.solve_button.pack(expand = True, fill = X, side = "left")
		self.check_button.pack(expand = True, fill = X, side = "left")
		self.generate_button.pack(expand = True, fill = X, side = "left")

	def reset_grid(self):
		self.can.delete(ALL)
		self.protected = 81 * [0]
		self.grille()

	def generate(self):
		self.board.generate(10)
		self.reset_grid()


	def get_coordonate(self, grid_x, grid_y):
		try:
			return (MARGIN_X + (grid_x * CELL_SIZE) + (CELL_SIZE / 2), MARGIN_Y + (grid_y * CELL_SIZE) + (CELL_SIZE / 2))
		except:
			return (-1, -1)

	def key_pressed(self, event):
		grid_x, grid_y = self.screen_to_grid(event.x, event.y)
		x, y = self.get_coordonate(grid_x, grid_y)
		self.can.focus_set()
		number = 0

		try:
			number = int(event.char)
		except:
			print("", end="")
		x0, y0, x1, y1 = self.can.coords(self.can.find_withtag("cursor"))

		if  x != -1 and y != -1 and self.board.can_insert(grid_x, grid_y, number) and x >= x0 and x <= x1 and y >= y0 and y <= y1 and number != 0 and \
			self.protected[CELL_NUMBER * grid_y + grid_x] == 0:
				self.board.grid[grid_y][grid_x] = int(event.char)
				self.click(event)
				return

	def solve(self):
		if self.board.is_finish():
			print("Sudoku déjà résolu.")
		else:
			self.can.delete("protected")
			print("En cours de résolution...")
			if self.board.solve():
				self.fill()
				print("Terminé!")

	def validate(self):
		print(self.board.is_finish())

	def screen_to_grid(self, x, y):
		pos_x = (x - MARGIN_X) // CELL_SIZE
		if pos_x < 0 or pos_x >= CELL_NUMBER:
			pos_x = None
		pos_y = (y - MARGIN_Y) // CELL_SIZE
		if pos_y < 0 or pos_y >= CELL_NUMBER:
			pos_y = None
		return pos_x, pos_y

	def draw_cursor(self, evenement):
		self.can.focus_set()
		grid_x, grid_y = self.screen_to_grid(evenement.x, evenement.y)
		if grid_x is None or grid_y is None:
			return
		self.can.delete("cursor")
		color = "green"
		if self.protected[CELL_NUMBER * grid_y + grid_x] == 1:
			color = "black"
		x0 = MARGIN_X + grid_x * CELL_SIZE + 1
		y0 = MARGIN_Y + grid_y * CELL_SIZE + 1
		x1 = MARGIN_X + (grid_x + 1) * CELL_SIZE - 1
		y1 = MARGIN_Y + (grid_y + 1) * CELL_SIZE - 1
		self.can.create_rectangle(x0, y0, x1, y1, outline = color, tags = "cursor")

	def fill(self):
		for i in range(CELL_NUMBER):
			for j in range(CELL_NUMBER):
				if self.board.grid[i][j] != 0:
					x, y = self.get_coordonate(j, i)
					self.can.create_text(x, y, text = self.board.grid[i][j], tags = "protected", fill = "black")
					self.protected[CELL_NUMBER * i + j] = 1

	def click(self, evenement):
		grid_x, grid_y = self.screen_to_grid(evenement.x, evenement.y)
		x, y = self.get_coordonate(grid_x, grid_y)
		tag = str(x) + str(y)
		self.can.delete(tag)
		self.can.create_text(x, y, text=self.board.grid[grid_y][grid_x], tags=tag, fill = "purple")

	def grille(self):
		y0 = 10
		x0 = 15

		for i in range(CELL_NUMBER + 1):
			width = 1
			if i % 3 == 0:
				width = 3
			self.can.create_line(x0 + CELL_SIZE * i, y0, x0 + CELL_SIZE * i, y0 + CELL_NUMBER * CELL_SIZE, width = width, fill = "black")
			self.can.create_line(x0, y0 + CELL_SIZE * i, x0 + CELL_NUMBER * CELL_SIZE, y0 + CELL_SIZE * i, width = width, fill = "black")

		self.fill()

class Grid:
	def __init__(self, grid = None):
		self.grid = grid
		if grid is None:
			self.generate(10)

	def __str__(self):
		grille = ""
		for i in range(CELL_NUMBER):
			if (i % 3 == 0):
				grille += "+-------"*3 + "+" + "\n"
			grille += "| "
			for j in range(CELL_NUMBER):
				if (j != 0 and j % 3 == 0):
					grille += "| "
				grille += str(self.grid[i][j]) + " "
			grille += "|" + "\n"
		grille += "+-------"*3 + "+" + "\n"

		return grille

	def play(self):
		while True:
			print("(C)heck (L)ine - (C)heck (C)olumn - (C)heck (S)quare - (S)olve - (F)inish?")
			var = input()
			if var == "S":
				print("Resolution...\n")
				self.solve()
			elif var == "CL" or var == "CC" or var == "CS":
				n = int(input("Number: "))
				if var == "CL":
					i = int(input("Line: "))
					print(self.check_row(i, n))
				elif var == "CC":
					j = int(input("Column: "))
					print(self.check_column(j, n))
				elif var == "CS":
					i = int(input("Line: "))
					j = int(input("Column: "))
					print(self.check_square(i, j, n))
			elif var == "F":
				print(self.is_finish())
			print(self)
			if self.is_finish():
				break
	
	def can_insert(self, row, column, digit):
		ret = palier_3.can_insert(self.grid, row, column, digit)
		return True if ret is None else ret

	def generate(self, number):
		self.grid = [[ 0 for col in range(CELL_NUMBER)] for row in range(CELL_NUMBER)]
		n = 0

		while n < number and not self.is_finish():
			i = random.randint(0, 8)
			j = random.randint(0, 8)
			k = random.randint(1, 9)
			if self.grid[i][j] == 0 and self.can_insert(i, j, k):
				self.grid[i][j] = k
				n = n + 1

	def is_finish(self):
		ret = palier_3.is_finished(self.grid)
		return False if ret is None else ret

	def solve(self):
		ret = palier_4.solve(self.grid)
		return True if ret is None else ret

def parse(file):
	if file is None:
		return None
	contents = open(file).read()
	tab =[item.split() for item in contents.split('\n')[:-1]]
	tab = [[int(elt) for elt in inner] for inner in tab]
	return tab

parser = argparse.ArgumentParser()
#parser.add_argument("-t", "--test", help = "lance la moulinette de test", action = "store_true")
#parser.add_argument("-d", "--debug", help = "désactive l'interface graphique", action = "store_true")
parser.add_argument("-f", "--file", nargs = '?', default = None, help = "mettre un sudoku manuellement")
args = parser.parse_args()
g = Grid(parse(args.file))

#if args.test:
#	g1 = Grid(parse("finish.txt"))
#	g2 = Grid(parse("simple.txt"))
#	g3 = Grid(parse("diabolique.txt"))
#	moulinette.test(g1, g2, g3)
#elif args.debug:
#	moulinette.test()
#	print(g)
#	g.play()
#else:
window = Tk()
window.title("SupBio Solver")
interface = Interface(window, g)
interface.can.pack(side = LEFT)
interface.grille()
interface.mainloop()
