import sys
import traceback
import palier_1
import palier_2
import palier_3
import palier_4

successes = 0
fails = 0

def success(name):
	global successes
	successes += 1
	print(name + ': OK')

def fail(name):
	global fails
	fails += 1
	_, _, tb = sys.exc_info()
	tb_info = traceback.extract_tb(tb)
	filename, line, func, text = tb_info[-1]
	print(name + ': Failed to {}'.format(text))

def test_can_insert_in_row():
	try:
		assert palier_1.can_insert_in_row([5, 3, 0, 0, 7, 0, 0, 0, 0], 1) == True
		success('can_insert_in_row 1')
	except AssertionError:
		fail('can_insert_in_row 1')
	try:
		assert palier_1.can_insert_in_row([5, 3, 0, 0, 7, 0, 0, 0, 0], 3) == False
		success('can_insert_in_row 1')
	except AssertionError:
		fail('can_insert_in_row 1')

def test_can_insert_in_column():
	try:
		assert palier_1.can_insert_in_column([5, 6, 0, 8, 4, 7, 0, 0, 0], 2) == True
		success('can_insert_in_column 1')
	except AssertionError:
		fail('can_insert_in_column 1')
	try:
		assert palier_1.can_insert_in_column([5, 6, 0, 8, 4, 7, 0, 0, 0], 6) == False
		success('can_insert_in_column 2')
	except AssertionError:
		fail('can_insert_in_column 2')

def test_can_insert_in_square():
	try:
		assert palier_1.can_insert_in_square([[5, 3, 0], [6, 0, 0], [0, 9, 8]], 1) == True
		success('can_insert_in_square 1')
	except AssertionError:
		fail('can_insert_in_square 1')
	try:
		assert palier_1.can_insert_in_square([[5, 3, 0], [6, 0, 0], [0, 9, 8]], 9) == False
		success('can_insert_in_square 2')
	except AssertionError:
		fail('can_insert_in_square 2')

def test_get_row(grid):
	try:
		assert palier_2.get_row(grid, 1) == [6, 0, 0, 1, 9, 5, 0, 0, 0]
		success('get_row 1')
	except AssertionError:
		fail('get_row 1')

def test_get_grid(grid):
	try:
		assert palier_2.get_column(grid, 8) == [0, 0, 0, 3, 1, 6, 0, 5, 9]
		success('get_grid 1')
	except AssertionError:
		fail('get_grid 1')

def test_get_square(grid):
	try:
		assert palier_2.get_square(grid, 0, 2) == [[0, 6, 0], [0, 0, 0], [0, 0, 0]]
		success('get_square 1')
	except AssertionError:
		fail('get_square 1')

def test_can_insert(grid):
	try:
		assert palier_3.can_insert(grid, 2, 0, 2) == True
		success('can_insert 1')
	except AssertionError:
		fail('can_insert 1')
	try:
		assert palier_3.can_insert(grid, 2, 0, 5) == False
		success('can_insert 2')
	except AssertionError:
		fail('can_insert 2')

def test_is_valid(grid):
	try:
		assert palier_3.is_valid(grid) == True
		success('is_valid with good grid')
	except AssertionError:
		fail('is_valid with good grid')
	grid[0][2] = 5
	try:
		assert palier_3.is_valid(grid) == False
		success('is_valid with bad grid')
	except AssertionError:
		fail('is_valid with bad grid')

def test_is_filled(grid):
	try:
		assert palier_3.is_filled(grid) == False
		success('is_filled 1')
	except AssertionError:
		fail('is_filled 1')

def test_is_finished(grid):
	try:
		assert palier_3.is_finished(grid) == False
		success('is_finished 1')
	except AssertionError:
		fail('is_finished 1')

def test_solve(grid):
	palier_4.solve(grid)
	try:
		assert palier_3.is_finished(grid) == True
		success('is_finished after solve')
	except AssertionError:
		fail('is_finished after solve')

def test_threshold_1():
	print('THRESHOLD 1')
	test_can_insert_in_row()
	test_can_insert_in_column()
	test_can_insert_in_square()

def test_threshold_2():
	print('')
	print('THRESHOLD 2')
	grid = [
		[5, 3, 0, 0, 7, 0, 0, 0, 0],
		[6, 0, 0, 1, 9, 5, 0, 0, 0],
		[0, 9, 8, 0, 0, 0, 0, 6, 0],
		[8, 0, 0, 0, 6, 0, 0, 0, 3],
		[4, 0, 0, 8, 0, 3, 0, 0, 1],
		[7, 0, 0, 0, 2, 0, 0, 0, 6],
		[0, 6, 0, 0, 0, 0, 2, 8, 0],
		[0, 0, 0, 4, 1, 9, 0, 0, 5],
		[0, 0, 0, 0, 8, 0, 0, 7, 9]
	]
	test_get_row(grid)
	test_get_grid(grid)
	test_get_square(grid)

def test_threshold_3():
	print('')
	print('THRESHOLD 3')
	grid = [
		[5, 3, 0, 0, 7, 0, 0, 0, 0],
		[6, 0, 0, 1, 9, 5, 0, 0, 0],
		[0, 9, 8, 0, 0, 0, 0, 6, 0],
		[8, 0, 0, 0, 6, 0, 0, 0, 3],
		[4, 0, 0, 8, 0, 3, 0, 0, 1],
		[7, 0, 0, 0, 2, 0, 0, 0, 6],
		[0, 6, 0, 0, 0, 0, 2, 8, 0],
		[0, 0, 0, 4, 1, 9, 0, 0, 5],
		[0, 0, 0, 0, 8, 0, 0, 7, 9]
	]
	test_can_insert(grid)
	test_is_valid(grid)
	test_is_filled(grid)
	test_is_finished(grid)

def test_threshold_4():
	print('')
	print('THRESHOLD 4')
	grid = [
		[5, 3, 0, 0, 7, 0, 0, 0, 0],
		[6, 0, 0, 1, 9, 5, 0, 0, 0],
		[0, 9, 8, 0, 0, 0, 0, 6, 0],
		[8, 0, 0, 0, 6, 0, 0, 0, 3],
		[4, 0, 0, 8, 0, 3, 0, 0, 1],
		[7, 0, 0, 0, 2, 0, 0, 0, 6],
		[0, 6, 0, 0, 0, 0, 2, 8, 0],
		[0, 0, 0, 4, 1, 9, 0, 0, 5],
		[0, 0, 0, 0, 8, 0, 0, 7, 9]
	]
	test_solve(grid)

test_threshold_1()
test_threshold_2()
test_threshold_3()
test_threshold_4()

print('')
print(str(successes) + '/' + str(successes + fails) + ' tests passed')
input()